@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/exim_monitor/em_globals.c,v 1.5 2005/01/25 14:16:33 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/exim_monitor/em_globals.c,v 1.7 2005/05/23 15:28:38 fanf2 Exp $ */
 
 /*************************************************
 *                Exim Monitor                    *
@@ -52,7 +52,7 @@ uschar *bmi_verdicts           = NULL;
 int     body_max = 20000;
 
 uschar *exim_path              = US BIN_DIRECTORY "/exim"
-			"\0<---------------Space to patch exim_path->";
+                        "\0<---------------Space to patch exim_path->";
 
 int     eximon_initialized = FALSE;
 
@@ -135,10 +135,17 @@ BOOL    deliver_firsttime      = FALSE;
 BOOL    deliver_freeze         = FALSE;
 int     deliver_frozen_at      = 0;
 BOOL    deliver_manual_thaw    = FALSE;
+
+#ifdef EXPERIMENTAL_DOMAINKEYS
+uschar *dk_signing_domain      = NULL;
+uschar *dk_signing_selector    = NULL;
+int     dk_do_verify           = 0;
+#endif
+
 BOOL    dont_deliver           = FALSE;
 
 #ifdef WITH_CONTENT_SCAN
-BOOL   fake_reject             = FALSE;
+int     fake_response          = OK;
 #endif
 
 header_line *header_last       = NULL;
