@@ -1,14 +1,28 @@
+/* $Cambridge: exim/exim-src/src/auths/cram_md5.c,v 1.2 2005/01/04 10:00:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
+
+/* The stand-alone version just tests the algorithm. We have to drag
+in the MD5 computation functions, without their own stand-alone main
+program. */
+
+#ifdef STAND_ALONE
+#define CRAM_STAND_ALONE
+#include "md5.c"
+
+
+/* This is the normal, non-stand-alone case */
+
+#else
 #include "../exim.h"
 #include "cram_md5.h"
 
-
 /* Options specific to the cram_md5 authentication mechanism. */
 
 optionlist auth_cram_md5_options[] = {
@@ -56,6 +70,8 @@ if (ob->client_secret != NULL)
   }
 }
 
+#endif  /* STAND_ALONE */
+
 
 
 /*************************************************
@@ -79,11 +95,11 @@ Returns:         nothing
 */
 
 static void
-compute_cram_md5(char *secret, char *challenge, uschar *digestptr)
+compute_cram_md5(uschar *secret, uschar *challenge, uschar *digestptr)
 {
 md5 base;
 int i;
-int len = (int)strlen(secret);
+int len = Ustrlen(secret);
 uschar isecret[64];
 uschar osecret[64];
 uschar md5secret[16];
@@ -95,7 +111,7 @@ if (len > 64)
   {
   md5_start(&base);
   md5_end(&base, (uschar *)secret, len, md5secret);
-  secret = (char *)md5secret;
+  secret = (uschar *)md5secret;
   len = 16;
   }
 
@@ -104,8 +120,7 @@ versions. */
 
 memcpy(isecret, secret, len);
 memset(isecret+len, 0, 64-len);
-memcpy(osecret, secret, len);
-memset(osecret+len, 0, 64-len);
+memcpy(osecret, isecret, 64);
 
 for (i = 0; i < 64; i++)
   {
@@ -117,7 +132,7 @@ for (i = 0; i < 64; i++)
 
 md5_start(&base);
 md5_mid(&base, isecret);
-md5_end(&base, (uschar *)challenge, (int)strlen(challenge), md5secret);
+md5_end(&base, (uschar *)challenge, Ustrlen(challenge), md5secret);
 
 /* Compute the outer MD5 digest */
 
@@ -127,6 +142,7 @@ md5_end(&base, md5secret, 16, digestptr);
 }
 
 
+#ifndef STAND_ALONE
 
 /*************************************************
 *             Server entry point                 *
@@ -135,13 +151,13 @@ md5_end(&base, md5secret, 16, digestptr);
 /* For interface, see auths/README */
 
 int
-auth_cram_md5_server(auth_instance *ablock, char *data)
+auth_cram_md5_server(auth_instance *ablock, uschar *data)
 {
 auth_cram_md5_options_block *ob =
   (auth_cram_md5_options_block *)(ablock->options_block);
-char *challenge = string_sprintf("<%d.%d@%s>", getpid(), time(NULL),
+uschar *challenge = string_sprintf("<%d.%d@%s>", getpid(), time(NULL),
   primary_hostname);
-char *clear, *secret;
+uschar *clear, *secret;
 uschar digest[16];
 int i, rc, len;
 
@@ -149,7 +165,7 @@ int i, rc, len;
 an example string taken from the RFC. */
 
 if (running_in_test_harness)
-  challenge = "<1896.697170952@postoffice.reston.mci.net>";
+  challenge = US"<1896.697170952@postoffice.reston.mci.net>";
 
 /* No data should have been sent with the AUTH command */
 
@@ -157,7 +173,7 @@ if (*data != 0) return UNEXPECTED;
 
 /* Send the challenge, read the return */
 
-if ((rc = auth_get_data(&data, challenge)) != OK) return rc;
+if ((rc = auth_get_data(&data, challenge, Ustrlen(challenge))) != OK) return rc;
 if ((len = auth_b64decode(data, &clear)) < 0) return BAD64;
 
 /* The return consists of a user name, space-separated from the CRAM-MD5
@@ -165,8 +181,8 @@ digest, expressed in hex. Extract the user name and put it in $1. Then check
 that the remaining length is 32. */
 
 expand_nstring[1] = clear;
-while (*clear != 0 && !isspace((uschar)*clear)) clear++;
-if (!isspace((uschar)*clear)) return FAIL;
+while (*clear != 0 && !isspace(*clear)) clear++;
+if (!isspace(*clear)) return FAIL;
 *clear++ = 0;
 
 expand_nlength[1] = clear - expand_nstring[1] - 1;
@@ -176,6 +192,7 @@ expand_nmax = 1;
 /* Expand the server_secret string so that it can compute a value dependent on
 the user name if necessary. */
 
+debug_print_string(ablock->server_debug_string);    /* customized debugging */
 secret = expand_string(ob->server_secret);
 
 /* A forced fail implies failure of authentication - i.e. we have no secret for
@@ -192,6 +209,17 @@ if (secret == NULL)
 
 compute_cram_md5(secret, challenge, digest);
 
+HDEBUG(D_auth)
+  {
+  uschar buff[64];
+  debug_printf("CRAM-MD5: user name = %s\n", expand_nstring[1]);
+  debug_printf("          challenge = %s\n", challenge);
+  debug_printf("          received  = %s\n", clear);
+  Ustrcpy(buff,"          digest    = ");
+  for (i = 0; i < 16; i++) sprintf(CS buff+22+2*i, "%02x", digest[i]);
+  debug_printf("%.54s\n", buff);
+  }
+
 /* We now have to compare the digest, which is 16 bytes in binary, with the
 data received, which is expressed in lower case hex. We checked above that
 there were 32 characters of data left. */
@@ -221,23 +249,23 @@ auth_cram_md5_client(
   smtp_inblock *inblock,                 /* input connection */
   smtp_outblock *outblock,               /* output connection */
   int timeout,                           /* command timeout */
-  char *buffer,                          /* for reading response */
+  uschar *buffer,                          /* for reading response */
   int buffsize)                          /* size of buffer */
 {
 auth_cram_md5_options_block *ob =
   (auth_cram_md5_options_block *)(ablock->options_block);
-char *secret = expand_string(ob->client_secret);
-char *name = expand_string(ob->client_name);
-char *challenge, *p;
+uschar *secret = expand_string(ob->client_secret);
+uschar *name = expand_string(ob->client_name);
+uschar *challenge, *p;
 int i;
 uschar digest[16];
 
-/* If expansion of either the secret or the user name failed, return FORCEFAIL
+/* If expansion of either the secret or the user name failed, return CANCELLED
 or ERROR, as approriate. */
 
 if (secret == NULL || name == NULL)
   {
-  if (expand_string_forcedfail) return FORCEFAIL;
+  if (expand_string_forcedfail) return CANCELLED;
   string_format(buffer, buffsize, "expansion of \"%s\" failed in "
     "%s authenticator: %s",
     (secret == NULL)? ob->client_secret : ob->client_name,
@@ -248,9 +276,9 @@ if (secret == NULL || name == NULL)
 /* Initiate the authentication exchange and read the challenge, which arrives
 in base 64. */
 
-if (!smtp_write_command(outblock, FALSE, "AUTH %s\r\n", ablock->public_name))
+if (smtp_write_command(outblock, FALSE, "AUTH %s\r\n", ablock->public_name) < 0)
   return FAIL_SEND;
-if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
+if (smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout) < 0)
   return FAIL;
 
 if (auth_b64decode(buffer + 4, &challenge) < 0)
@@ -273,7 +301,7 @@ while (*p != 0) p++;
 
 for (i = 0; i < 16; i++)
   {
-  sprintf(p, "%02x", digest[i]);
+  sprintf(CS p, "%02x", digest[i]);
   p += 2;
   }
 
@@ -282,11 +310,38 @@ in big_buffer, but auth_b64encode() returns its result in working store,
 so calling smtp_write_command(), which uses big_buffer, is OK. */
 
 buffer[0] = 0;
-if (!smtp_write_command(outblock, FALSE, "%s\r\n", auth_b64encode(big_buffer,
-  p - big_buffer))) return FAIL_SEND;
+if (smtp_write_command(outblock, FALSE, "%s\r\n", auth_b64encode(big_buffer,
+  p - big_buffer)) < 0) return FAIL_SEND;
 
 return smtp_read_response(inblock, (uschar *)buffer, buffsize, '2', timeout)?
   OK : FAIL;
 }
+#endif  /* STAND_ALONE */
+
+
+/*************************************************
+**************************************************
+*             Stand-alone test program           *
+**************************************************
+*************************************************/
+
+#ifdef STAND_ALONE
+
+int main(int argc, char **argv)
+{
+int i;
+uschar *secret = US argv[1];
+uschar *challenge = US argv[2];
+uschar digest[16];
+
+compute_cram_md5(secret, challenge, digest);
+
+for (i = 0; i < 16; i++) printf("%02x", digest[i]);
+printf("\n");
+
+return 0;
+}
+
+#endif
 
 /* End of cram_md5.c */
