@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/auths/md5.c,v 1.2 2005/01/04 10:00:43 ph10 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2005 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #ifndef STAND_ALONE
@@ -13,6 +15,7 @@ to be able to do I/O */
 
 #else
 #include <stdio.h>
+#include "../mytypes.h"
 typedef struct md5 {
   unsigned int length;
   unsigned int abcd[4];
@@ -301,11 +304,11 @@ for (i = 0; i < 4; i++)
 **************************************************
 *************************************************/
 
-#ifdef STAND_ALONE
+#if defined STAND_ALONE & !defined CRAM_STAND_ALONE
 
 /* Test values */
 
-unsigned static char *tests[] = {
+static uschar *tests[] = {
   "", "d41d8cd98f00b204e9800998ecf8427e",
 
   "a", "0cc175b9c0f1b6a831c399e269772661",
@@ -333,18 +336,18 @@ int main(void)
 {
 md5 base;
 int i = 0x01020304;
-char *ctest = (char *)(&i);
+uschar *ctest = (uschar *)(&i);
 uschar buffer[256];
 uschar digest[16];
 printf("Checking md5: %s-endian\n", (ctest[0] == 0x04)? "little" : "big");
 
-for (i = 0; i < sizeof(tests)/sizeof(char *); i += 2)
+for (i = 0; i < sizeof(tests)/sizeof(uschar *); i += 2)
   {
   int j;
-  char s[33];
+  uschar s[33];
   printf("%s\nShould be: %s\n", tests[i], tests[i+1]);
   md5_start(&base);
-  md5_end(&base, tests[i], (int)strlen(tests[i]), digest);
+  md5_end(&base, tests[i], strlen(tests[i]), digest);
   for (j = 0; j < 16; j++) sprintf(s+2*j, "%02x", digest[j]);
   printf("Computed:  %s\n", s);
   if (strcmp(s, tests[i+1]) != 0) printf("*** No match ***\n");
