@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/buildconfig.c,v 1.6 2005/02/17 11:58:25 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/buildconfig.c,v 1.10 2005/06/27 14:29:43 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -15,8 +15,13 @@
 /* This auxiliary program builds the file config.h by the following
 process:
 
-First it reads Makefile, looking for certain OS-specific definitions which it
-uses to define macros. Then it reads the defaults file config.h.defaults.
+First, it determines the size of off_t and time_t variables, and generates
+macro code to define OFF_T_FMT and TIME_T_FMT as suitable formats, if they are
+not already defined in the system-specific header file.
+
+Then it reads Makefile, looking for certain OS-specific definitions which it
+uses to define some specific macros. Finally, it reads the defaults file
+config.h.defaults.
 
 The defaults file contains normal C #define statements for various macros; if
 the name of a macro is found in the environment, the environment value replaces
@@ -96,6 +101,8 @@ if (!OK)
 int
 main(int argc, char **argv)
 {
+off_t test_off_t = 0;
+time_t test_time_t = 0;
 FILE *base;
 FILE *new;
 int last_initial = 'A';
@@ -132,13 +139,49 @@ fprintf(new, "using values specified in the configuration file Local/Makefile.\n
 fprintf(new, "Do not edit it. Instead, edit Local/Makefile and "
   "rerun make. */\n\n");
 
-/* First, search the makefile for certain settings */
+/* First, deal with the printing format for off_t variables. We assume that if
+the size of off_t is greater than 4, "%lld" will be available as a format for
+printing long long variables, and there will be support for the long long type.
+This assumption is known to be OK for the common operating systems. */
+
+fprintf(new, "#ifndef OFF_T_FMT\n");
+if (sizeof(test_off_t) > 4)
+  {
+  fprintf(new, "#define OFF_T_FMT  \"%%lld\"\n");
+  fprintf(new, "#define LONGLONG_T long long int\n");
+  }
+else
+  {
+  fprintf(new, "#define OFF_T_FMT  \"%%ld\"\n");
+  fprintf(new, "#define LONGLONG_T long int\n");
+  }
+fprintf(new, "#endif\n\n");
+
+/* Now do the same thing for time_t variables. If the length is greater than
+4, we want to assume long long support (even if off_t was less than 4). If the
+length is 4 or less, we can leave LONGLONG_T to whatever was defined above for
+off_t. */
+
+fprintf(new, "#ifndef TIME_T_FMT\n");
+if (sizeof(test_time_t) > 4)
+  {
+  fprintf(new, "#define TIME_T_FMT  \"%%lld\"\n");
+  fprintf(new, "#undef  LONGLONG_T\n");
+  fprintf(new, "#define LONGLONG_T long long int\n");
+  }
+else
+  {
+  fprintf(new, "#define TIME_T_FMT  \"%%ld\"\n");
+  }
+fprintf(new, "#endif\n\n");
+
+/* Now search the makefile for certain settings */
 
 base = fopen("Makefile", "rb");
 if (base == NULL)
   {
   printf("*** Buildconfig: failed to open Makefile\n");
-  fclose(new);
+  (void)fclose(new);
   exit(1);
   }
 
@@ -250,7 +293,7 @@ if (strcmp(cc, "gcc") == 0 && strstr(ostype, "IRIX") != NULL)
   }
 
 fprintf(new, "\n");
-fclose(base);
+(void)fclose(base);
 
 
 /* Now handle the macros listed in the defaults */
@@ -259,7 +302,7 @@ base = fopen("../src/config.h.defaults", "rb");
 if (base == NULL)
   {
   printf("*** Buildconfig: failed to open ../src/config.h.defaults\n");
-  fclose(new);
+  (void)fclose(new);
   exit(1);
   }
 
@@ -677,6 +720,7 @@ while (fgets(buffer, sizeof(buffer), base) != NULL)
     else if (strcmp(name, "RADIUS_LIB_TYPE") == 0)
       {
       if (strcmp(value, "RADIUSCLIENT") == 0 ||
+          strcmp(value, "RADIUSCLIENTNEW") == 0 ||
           strcmp(value, "RADLIB") == 0)
         {
         fprintf(new, "#define RADIUS_LIB_%s\n", value);
@@ -771,7 +815,7 @@ while (fgets(buffer, sizeof(buffer), base) != NULL)
     }
   }
 
-fclose(base);
+(void)fclose(base);
 
 /* If any AUTH macros were defined, ensure that SUPPORT_CRYPTEQ is also
 defined. */
@@ -785,7 +829,7 @@ if (have_auth)
 /* End off */
 
 fprintf(new, "\n/* End of config.h */\n");
-fclose(new);
+(void)fclose(new);
 return 0;
 }
 
