@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/demime.c,v 1.4 2005/02/17 11:58:26 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/demime.c,v 1.7 2005/07/01 10:49:02 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -60,7 +60,7 @@ int demime(uschar **listptr) {
   if (!demime_ok)
     demime_rc = mime_demux(mbox_file, defer_error_buffer);
 
-  fclose(mbox_file);
+  (void)fclose(mbox_file);
 
   if (demime_rc == DEFER) {
     /* temporary failure (DEFER => DEFER) */
@@ -256,7 +256,7 @@ int mime_get_dump_file(uschar *extension, FILE **f, uschar *info) {
   }
   while(result != -1);
 
-  *f = fopen(CS file_name,"w+");
+  *f = fopen(CS file_name,"wb+");
   if (*f == NULL) {
     /* cannot open new dump file, disk full ? -> soft error */
     snprintf(CS info, 1024,"unable to open dump file");
@@ -346,7 +346,7 @@ int mime_read_line(FILE *f, int mime_demux_mode, uschar *buffer, long *num_copie
           c = fgetc(f);
           if (c == EOF) break;
           if ( (c == '\t') || (c == ' ') ) continue;
-          ungetc(c,f);
+          (void)ungetc(c,f);
         };
         /* end of the header, terminate with ';' */
         c = ';';
@@ -452,7 +452,7 @@ int mime_check_boundary(uschar *line, struct boundary *boundaries) {
     for (i = 2; i < Ustrlen(line); i++) {
       if ((line[i] != ' ') && (line[i] != '\t')) {
         workbuf[j] = line[i];
-	      j++;
+        j++;
       };
     };
     workbuf[j+1]='\0';
@@ -1078,14 +1078,14 @@ int mime_demux(FILE *f, uschar *info) {
          if (data_len == -2) {
            /* temp error, turn off uudecode mode */
            if (uu_dump_file != NULL) {
-            fclose(uu_dump_file); uu_dump_file = NULL;
+            (void)fclose(uu_dump_file); uu_dump_file = NULL;
            };
            uu_mode = MIME_UU_MODE_OFF;
            return DEFER;
          }
          else if (data_len == -1) {
            if (uu_dump_file != NULL) {
-            fclose(uu_dump_file); uu_dump_file = NULL;
+            (void)fclose(uu_dump_file); uu_dump_file = NULL;
            };
            uu_mode = MIME_UU_MODE_OFF;
            data_len = 0;
@@ -1101,7 +1101,7 @@ int mime_demux(FILE *f, uschar *info) {
           check for single "end" tag on line */
           if ((strncmpic(line,US"end",3) == 0) && (line[3] < 32)) {
             if (uu_dump_file != NULL) {
-              fclose(uu_dump_file); uu_dump_file = NULL;
+              (void)fclose(uu_dump_file); uu_dump_file = NULL;
             };
             uu_mode = MIME_UU_MODE_OFF;
           }
@@ -1110,7 +1110,7 @@ int mime_demux(FILE *f, uschar *info) {
             if (data_len == -2) {
                /* temp error, turn off uudecode mode */
                if (uu_dump_file != NULL) {
-                 fclose(uu_dump_file); uu_dump_file = NULL;
+                 (void)fclose(uu_dump_file); uu_dump_file = NULL;
                };
                uu_mode = MIME_UU_MODE_OFF;
                return DEFER;
@@ -1152,7 +1152,7 @@ int mime_demux(FILE *f, uschar *info) {
               mime_demux(mime_dump_file,info);
             };
 
-            fclose(mime_dump_file); mime_dump_file = NULL;
+            (void)fclose(mime_dump_file); mime_dump_file = NULL;
           };
         }
         else if (tmp == 2) {
@@ -1167,7 +1167,7 @@ int mime_demux(FILE *f, uschar *info) {
               mime_demux(mime_dump_file,info);
             };
 
-            fclose(mime_dump_file); mime_dump_file = NULL;
+            (void)fclose(mime_dump_file); mime_dump_file = NULL;
           };
         }
         else {
@@ -1218,9 +1218,9 @@ int mime_demux(FILE *f, uschar *info) {
 
   /* close files, they could still be open */
   if (mime_dump_file != NULL)
-    fclose(mime_dump_file);
+    (void)fclose(mime_dump_file);
   if (uu_dump_file != NULL)
-    fclose(uu_dump_file);
+    (void)fclose(uu_dump_file);
 
   /* release line buffer */
   free(line);
