@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/filtertest.c,v 1.4 2005/02/17 11:58:26 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/filtertest.c,v 1.6 2005/06/27 14:29:43 ph10 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -181,7 +181,7 @@ if (fstat(fd, &statbuf) != 0)
 
 filebuf = store_get(statbuf.st_size + 1);
 rc = read(fd, filebuf, statbuf.st_size);
-close(fd);
+(void)close(fd);
 
 if (rc != statbuf.st_size)
   {
@@ -271,7 +271,7 @@ if (is_system)
 else
   {
   yield = (filter_type == FILTER_SIEVE)?
-    sieve_interpret(filebuf, RDO_REWRITE, NULL, &generated, &error)
+    sieve_interpret(filebuf, RDO_REWRITE, NULL, NULL, NULL, &generated, &error)
     :
     filter_interpret(filebuf, RDO_REWRITE, &generated, &error);
   }
